#!/usr/bin/env lua

require 'Test.Assertion'

if not pcall(require, 'Silva') then
    skip_all 'no Silva'
end

plan(17)

local sme = require 'Silva'
local rotas = require 'Rotas'
rotas.http_methods = { 'GET', 'POST', 'PUT' }

local app = rotas{}
array_equals( app.ALL, { 'GET', 'POST', 'PUT' } )

app.GET[sme('/*.html', 'shell')] = function (path)
    return path
end
app.GET[sme'/foo{?query}'] = function (params)
    return params.query
end
app.ALL['/bar{?query}'] = function (params)
    return params.query
end

error_matches( function () app.GET[42] = false end,
        "key not callable" )

error_matches( function () app.GET['/foo{=bad}'] = false end,
        "operator for future extension " )

error_matches( function () app.PATCH['/foo{?query}'] = false end,
        "attempt to index " )

local fn, capture, res
fn, capture = app('GET', '/index.html')
is_function( fn )
res = fn(capture)
equals( res, '/index.html' )

fn, capture = app('GET', '/foo?query=42')
is_function( fn )
res = fn(capture)
equals( res, '42' )

fn, capture = app('GET', '/bar?query=42')
is_function( fn )
res = fn(capture)
equals( res, '42' )

fn, capture = app('PUT', '/bar?query=42')
is_function( fn )
res = fn(capture)
equals( res, '42' )

fn, capture = app('GET', '/baz?query=42')
is_nil( fn )
is_nil( capture )

fn, capture = app('FOO', '/foo')
is_nil( fn )
is_nil( capture )

error_matches( function () app('ALL', '/index.html') end,
        "allowed only for registration" )
